@@ -10,8 +10,11 @@ __description__ = (
     "CLI Command making OCI Bastion and kubernetes usage simple and fast"
 )
 
-__version__ = "1.6.4"
+__version__ = "1.9.2"
 __author__ = "Jiri Otoupal"
 __author_email__ = "jiri-otoupal@ips-database.eu"
 __license__ = "MIT"
 __url__ = "https://github.com/jiri-otoupal/abst"
+__pypi_repo__ = "https://pypi.org/project/abst/"
+
+__version_name__ = "Kitten w/ Style Faster"
